/**
 * Cecilia, an implementation of the Fractal component model in C.
 *
 * Copyright (C) 2007 ST Microelectronics, INRIA
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Matthieu Leclercq, Lionel Debroux.
 */

// START SNIPPET: Content
/**
 * This files defines the opaque data types passed as void * (Cecilia IDL "any")
 * between multiple components.
 */

/**
 * This structure defines what the request receiver passes to the scheduler.
 */
typedef struct {
    /**
     * Callback to request receiver, used by the worker function.
     * This callback is a way to emulate the anonymous inline Runnable instance of
     * the Julia Comanche application (see its source code).
     */
    void * requestReceiver;
    /**
     * Pointer to worker function (handleRequest). This is passed to
     * pthread_create as third argument (multithread scheduler), or called directly
     * (sequential scheduler).
     */
    void * (*workerFunctionPtr)(void *);
    /**
     * Worker function arguments. This is passed to pthread_create as fourth argument
     * (multithread scheduler) or directly to the worker function (sequential scheduler).
     */
    void * workerFunctionArgs;
} SchedulerArgs;

/**
 * This structure defines what the scheduler passes to the worker function.
 */
typedef struct {
    /**
     * Callback to request receiver, used by the worker function.
     */
    void * requestReceiver;
    /**
     * Worker function arguments. This is passed to pthread_create as fourth argument
     * (multithread scheduler) or directly to the worker function (sequential scheduler).
     */
    void * workerFunctionArgs;
} WorkerArgs;

/**
 * This structure defines what the request analyzer passes to the request dispatcher,
 * which passes it to the request handlers it is bound to.
 */
typedef struct {
    /**
     * Request string.
     */
    char * request;
    /**
     * File descriptor used by the request handlers to write the reply.
     */
    int fd;
} RequestHandlerArgs;
